﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Drawing;
using System.Drawing.Imaging;

namespace DataPara
{
    class Work
    {

        private void ProcessFilesSeq()
        {
            Console.WriteLine("\nProcessing the image data in a sequential manner\n");

            // Load up all *.jpg files, and make a new folder for the modified data.
            string[] files = Directory.GetFiles
                (@".\Pictures", "*.jpg", SearchOption.AllDirectories);
            string newDir = @".\ModifiedPicturesSeq";
            Directory.CreateDirectory(newDir);
            
            // Process the image data in a blocking manner.
            foreach (string currentFile in files)
            {
               string filename = Path.GetFileName(currentFile);
               using (Bitmap bitmap = new Bitmap(currentFile))
               {
                  bitmap.RotateFlip(RotateFlipType.Rotate180FlipNone);
                  bitmap.Save(Path.Combine(newDir, filename),ImageFormat.Jpeg);
                  Console.WriteLine(" {0} processed by thread {1}", filename,
                        Thread.CurrentThread.ManagedThreadId);
                  
               }
            }
        }

        private void ProcessFilesPar()
        {
            Console.WriteLine("\nProcessing the data in a parallel manner\n");

            // Load up all *.jpg files, and make a new folder for the modified data.
            string[] files = Directory.GetFiles
                (@".\Pictures", "*.jpg", SearchOption.AllDirectories);
            string newDir = @".\ModifiedPicturesPara";
            Directory.CreateDirectory(newDir);

            // Process the image data in a parallel manner
        Parallel.ForEach(files, currentFile =>
            {
                string filename = Path.GetFileName(currentFile);
                using (Bitmap bitmap = new Bitmap(currentFile))
                {
                    bitmap.RotateFlip(RotateFlipType.Rotate180FlipNone);
                    bitmap.Save(Path.Combine(newDir, filename), ImageFormat.Jpeg);
                    Console.WriteLine(" {0} processed by thread {1}", filename,
                                        Thread.CurrentThread.ManagedThreadId);

                }
            }
                        );
            
        }
        public void Run()
        {
            DateTime dt1;
            TimeSpan ts1, ts2;

            Console.WriteLine("Picture processing using {0} processors.\n",
                                System.Environment.ProcessorCount);
            dt1 = DateTime.Now;

            ProcessFilesSeq();

            ts1  = DateTime.Now - dt1;

            dt1 = DateTime.Now;

            ProcessFilesPar();

            ts2 = DateTime.Now - dt1;

            Console.Write("\nSeq done in ");
            Console.WriteLine(ts1.Seconds + "," + 
                              ts1.Milliseconds + " s\n");

            Console.Write("\nPar done in ");
            Console.WriteLine(ts2.Seconds + "," +
                              ts2.Milliseconds + " s\n");
        }
    }

    
    class Program
    {
        static void Main(string[] args)
        {
            new Work().Run();
        }
    }
}
