using System;
using System.Threading;

namespace ResetEvents
{

    class CalculateTest
    {
        static void Main()
        {
            Calculate calc = new Calculate();
            Console.WriteLine("Result = {0}.",
                calc.Result(234).ToString());
            Console.WriteLine("Result = {0}.",
                calc.Result(55).ToString());
        }
    }

    class Calculate
    {
        double baseNumber, firstTerm, secondTerm, thirdTerm;
        AutoResetEvent[] autoEvents;
        ManualResetEvent manualEvent;

        // Generate random numbers to simulate the actual calculations.
        Random randomGenerator;

        public Calculate()
        {
            autoEvents = new AutoResetEvent[]
            {
                new AutoResetEvent(false),
                new AutoResetEvent(false),
                new AutoResetEvent(false)
            };

            manualEvent = new ManualResetEvent(false);
        }

        void CalculateBase()    
        {
            baseNumber = randomGenerator.NextDouble();

            // Signal that baseNumber is ready.
            manualEvent.Set();
        }

        // The following CalculateX methods all perform the same
        // series of steps as commented in CalculateFirstTerm.

        void CalculateFirstTerm()   
        {
            // Perform a precalculation.
            double preCalc = randomGenerator.NextDouble();

            // Wait for baseNumber to be calculated.
            manualEvent.WaitOne();

            // Calculate the first term from preCalc and baseNumber.
            firstTerm = preCalc * baseNumber *
                randomGenerator.NextDouble();

            // Signal that the calculation is finished.
            autoEvents[0].Set();
        }

        void CalculateSecondTerm()      
        {
            double preCalc = randomGenerator.NextDouble();
            manualEvent.WaitOne();
            secondTerm = preCalc * baseNumber *
                randomGenerator.NextDouble();
            autoEvents[1].Set();
        }

        void CalculateThirdTerm()   
        {
            double preCalc = randomGenerator.NextDouble();
            manualEvent.WaitOne();
            thirdTerm = preCalc * baseNumber *
                randomGenerator.NextDouble();
            autoEvents[2].Set();
        }

        public double Result(int seed)
        {
            randomGenerator = new Random(seed);


            // Simultaneously calculate the terms.
            Thread t1 = new Thread(new ThreadStart(CalculateBase));
            Thread t2 = new Thread(new ThreadStart(CalculateFirstTerm));
            Thread t3 = new Thread(new ThreadStart(CalculateSecondTerm));
            Thread t4 = new Thread(new ThreadStart(CalculateThirdTerm));
            t4.Start();
            t2.Start();
            t1.Start();
            t3.Start();

            // Wait for all of the terms to be calculated.
            AutoResetEvent.WaitAll(autoEvents);

            // Reset the wait handle for the next calculation.
            manualEvent.Reset();

            return firstTerm + secondTerm + thirdTerm;
            
        }
    }
   

}